/***************************************************************************
 *   Copyright (C) 2004 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "bpconf.h"
#include "feeddialog.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qtimer.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <kiconloader.h>
#include <klistview.h>
#include <kdialog.h>
#include <ktimewidget.h>
#include <kurlcompletion.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kfiledialog.h>
#include <kdirselectdialog.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kprocess.h>
#include <ktempfile.h>

#include <kmainwindow.h>
#include <klocale.h>

BPConf::BPConf() : KMainWindow( 0, "MainWindow" )
{
   m_pMain = new QWidget( this );
   setCentralWidget( m_pMain );
   setCaption( i18n( "BashPodder" ) );

   m_pDirComplete = new KURLCompletion( KURLCompletion::DirCompletion );

   QVBoxLayout *top = new QVBoxLayout( m_pMain, KDialog::marginHint(), KDialog::spacingHint() );
   QHBoxLayout *hl = new QHBoxLayout( top );
   hl->addWidget( new QLabel( i18n( "Data Directory:" ), m_pMain ) );
   m_pDataDir = new KLineEdit( m_pMain );
   m_pDataDir->setURLDropsEnabled( true );
   m_pDataDir->setCompletionObject( m_pDirComplete, true );
   hl->addWidget( m_pDataDir );
   m_pDataDirButton = new KPushButton( SmallIcon( "fileopen" ), QString::null, m_pMain );
   hl->addWidget( m_pDataDirButton );
   hl = new QHBoxLayout( top );
   m_pFeeds = new KListView( m_pMain );
   m_pFeeds->addColumn( i18n( "Name" ) );
   m_pFeeds->addColumn( i18n( "URL" ) );
   hl->addWidget( m_pFeeds );
   QVBoxLayout *vl = new QVBoxLayout( hl );
   m_pFeedAdd = new KPushButton( i18n( "Add" ), m_pMain );
   m_pFeedMod = new KPushButton( i18n( "Modify" ), m_pMain );
   m_pFeedRem = new KPushButton( i18n( "Delete" ), m_pMain );
   vl->addWidget( m_pFeedAdd );
   vl->addWidget( m_pFeedMod );
   vl->addWidget( m_pFeedRem );
   vl->addStretch();
   hl = new QHBoxLayout( top );
   hl->addWidget( new QLabel( i18n( "Check every:" ), m_pMain ) );
   m_pCheckInterval = new QComboBox( m_pMain );
   m_pCheckInterval->insertItem( i18n( "Hour" ) );
   m_pCheckInterval->insertItem( i18n( "Day" ) );
   m_pCheckInterval->insertItem( i18n( "Week" ) );
   m_pCheckInterval->insertItem( i18n( "Month" ) );
   hl->addWidget( m_pCheckInterval );
   m_pCheckOnLabel = new QLabel( i18n( "on" ), m_pMain );
   m_pCheckOnLabel->hide();
   hl->addWidget( m_pCheckOnLabel );
   m_pCheckOnDay = new QComboBox( m_pMain );
   m_pCheckOnDay->insertItem( i18n( "Sunday" ) );
   m_pCheckOnDay->insertItem( i18n( "Monday" ) );
   m_pCheckOnDay->insertItem( i18n( "Tuesday" ) );
   m_pCheckOnDay->insertItem( i18n( "Wednesday" ) );
   m_pCheckOnDay->insertItem( i18n( "Thursday" ) );
   m_pCheckOnDay->insertItem( i18n( "Friday" ) );
   m_pCheckOnDay->insertItem( i18n( "Saturday" ) );
   m_pCheckOnDay->hide();
   hl->addWidget( m_pCheckOnDay );
   m_pCheckOnDate = new KLineEdit( m_pMain );
   m_pCheckOnDate->hide();
   hl->addWidget( m_pCheckOnDate );
   m_pCheckAtLabel = new QLabel( i18n( "at" ), m_pMain );
   hl->addWidget( m_pCheckAtLabel );
   m_pCheckTime = new KTimeWidget( QTime( 0, 0 ) ,m_pMain );
   hl->addWidget( m_pCheckTime );
   hl->addStretch();
   hl = new QHBoxLayout( top );
   m_pCheckNow = new KPushButton( i18n( "Check Now" ), m_pMain );
   hl->addWidget( m_pCheckNow );
   hl->addStretch();
   m_pApply = new KPushButton( i18n( "Apply" ), m_pMain );
   m_pApply->setEnabled( false );
   hl->addWidget( m_pApply );
   m_pReset = new KPushButton( i18n( "Reset" ), m_pMain );
   m_pReset->setEnabled( false );
   hl->addWidget( m_pReset );

   setupActions();
   setAutoSaveSettings();
   createGUI();

   connect( m_pDataDir, SIGNAL( textChanged( const QString& ) ), SLOT( slotDataChange() ) );
   connect( m_pDataDirButton, SIGNAL( clicked() ), SLOT( slotDataDir() ) );
   connect( m_pFeeds, SIGNAL( selectionChanged() ), SLOT( slotFeedSelect() ) );
   connect( m_pFeedAdd, SIGNAL( clicked() ), SLOT( slotAddFeed() ) );
   connect( m_pFeedMod, SIGNAL( clicked() ), SLOT( slotModFeed() ) );
   connect( m_pFeedRem, SIGNAL( clicked() ), SLOT( slotRemFeed() ) );
   connect( m_pCheckInterval, SIGNAL( activated( int ) ), SLOT( slotInterval( int ) ) );
   connect( m_pCheckNow, SIGNAL( clicked() ), SLOT( slotCheckNow() ) );
   connect( m_pApply, SIGNAL( clicked() ), SLOT( slotApply() ) );
   connect( m_pReset, SIGNAL( clicked() ), SLOT( slotReset() ) );

   loadOptions( instance()->config() );
   slotFeedSelect();
   m_checkNow = false;
   m_timeOut = FINISH;
}

void BPConf::saveProperties( KConfig *config )
{
   saveOptions( config );
}

void BPConf::readProperties( KConfig *config )
{
   loadOptions( config );
}

bool BPConf::queryClose()
{
   if( m_modified )
   {
      int ret = KMessageBox::warningYesNoCancel( 0, i18n( "The settings have been modified.\nDo you want to save your changes or discard them?" ),
                                       i18n( "Unsaved Changes" ), i18n( "&Save" ), i18n( "&Discard" ) );
      switch( ret )
      {
         case KMessageBox::Yes:
            slotApply();
            break;
         case KMessageBox::Cancel:
            return false;
            break;
      }
   }

   if( !m_pCheckNow->isEnabled() )
      KMessageBox::information( 0, i18n( "bashpodder is still gathering files from the feeds.\nIt will continue when you quit" ), i18n( "bashpodder" ) );

   return true;
}

void BPConf::fileNew()
{
   m_pDataDir->setText( "~/PodCasts/" );
   m_pFeeds->clear();
   m_pCheckInterval->setCurrentItem( 1 );
   slotInterval( 1 );
   m_pCheckTime->setTime( QTime( 0, 0 ) );
   setModified( false );
   slotFeedSelect();
}

void BPConf::importConfig()
{
   QString fileName = KFileDialog::getOpenFileName( QString::null, "*.conf|" + i18n( "BashPodder Configuration" ) );
   if ( !fileName.isEmpty() )
      loadFeeds( fileName );
}

void BPConf::exportConfig()
{
   QString fileName = KFileDialog::getSaveFileName( QString::null, "*.conf|" + i18n( "BashPodder Configuration" ) );
   if( !fileName.isEmpty() )
   {
      if ( QFile::exists( fileName ) )
      {
         if ( KMessageBox::warningContinueCancel( 0,
              i18n ("A document called \"%1\" already exists.\nDo you want to overwrite it?").arg( fileName ),
              QString::null, i18n ("Overwrite") ) == KMessageBox::Cancel )
            return;
      }

      saveFeeds( fileName );
   }
}

void BPConf::slotDataDir()
{
   KURL newDir = KDirSelectDialog::selectDirectory( m_pDataDir->text(), false, this, i18n( "Select Data Directory" ) );
   if( !newDir.isEmpty() )
   {
      m_pDataDir->setText( newDir.path( +1 ) );
      setModified( true );
   }
}

void BPConf::slotFeedSelect()
{
   QListViewItem *item = m_pFeeds->selectedItem();
   if( item )
   {
      m_pFeedMod->setEnabled( true );
      m_pFeedRem->setEnabled( true );
   }
   else
   {
      m_pFeedMod->setEnabled( false );
      m_pFeedRem->setEnabled( false );
   }
}

void BPConf::slotAddFeed()
{
   FeedDialog *fd = new FeedDialog( this );
   int code = fd->exec();
   if( code == QDialog::Accepted )
   {
      new QListViewItem( m_pFeeds, fd->name(), fd->url() );
      setModified( true );
   }
}

void BPConf::slotModFeed()
{
   QListViewItem *item = m_pFeeds->selectedItem();
   if( !item )
   {
      slotFeedSelect();
      return;
   }

   FeedDialog *fd = new FeedDialog( this, item->text( 0 ), item->text( 1 ) );
   int code = fd->exec();
   if( code == QDialog::Accepted )
   {
      item->setText( 0, fd->name() );
      item->setText( 1, fd->url() );
      setModified( true );
   }
}

void BPConf::slotRemFeed()
{
   QListViewItem *item = m_pFeeds->selectedItem();
   if( !item )
   {
      slotFeedSelect();
      return;
   }

   delete item;
   slotFeedSelect();
}

void BPConf::slotInterval( int item )
{
   switch( item )
   {
      case 0:
         m_pCheckOnLabel->hide();
         m_pCheckOnDay->hide();
         m_pCheckOnDate->hide();
         m_pCheckAtLabel->hide();
         m_pCheckTime->hide();
         break;
      case 1:
         m_pCheckOnLabel->hide();
         m_pCheckOnDay->hide();
         m_pCheckOnDate->hide();
         m_pCheckAtLabel->show();
         m_pCheckTime->show();
         break;
      case 2:
         m_pCheckOnLabel->show();
         m_pCheckOnDay->show();
         m_pCheckOnDate->hide();
         m_pCheckAtLabel->show();
         m_pCheckTime->show();
         break;
      case 3:
         m_pCheckOnLabel->show();
         m_pCheckOnDay->hide();
         m_pCheckOnDate->show();
         m_pCheckAtLabel->show();
         m_pCheckTime->show();
         break;
   }
   setModified( true );
}

void BPConf::slotCheckNow()
{
   if( m_modified )
   {
      if( KMessageBox::questionYesNo( 0, i18n( "Do you wish to apply the changes before checking?" ), i18n( "Apply Changes" ) ) == KMessageBox::Yes )
      {
         slotApply();
         m_checkNow = true;
         return;
      }
   }

   QString exec = KURLCompletion::replacedPath( m_pDataDir->text(), true ) + "bashpodder.shell";
   if( !QFile::exists( exec ) )
   {
      KMessageBox::sorry( 0, i18n( "I can't find the current shell file %1.\nPerhaps its been deleted?" ).arg( exec ), i18n( "Check Now" ) );
      return;
   }
   KProcess *proc = new KProcess;
   *proc << exec;
   connect( proc,SIGNAL( processExited( KProcess* ) ), SLOT( slotCheckNowFinish( KProcess* ) ) );
   m_pCheckNow->setEnabled( false );
   proc->start( KProcess::OwnGroup );
}

void BPConf::slotCheckNowFinish( KProcess* )
{
   KMessageBox::information( 0, i18n( "Finished checking podcasts" ), i18n( "bashpodder finished" ) );
   m_pCheckNow->setEnabled( true );
}

void BPConf::slotApply()
{
   m_pMain->setEnabled( false );
   QString path = KURLCompletion::replacedPath( m_pDataDir->text(), true );
   QTimer *timer = new QTimer( this );
   connect( timer, SIGNAL( timeout() ), SLOT( slotTimeOut() ) );
   m_timeOut = MKDIR;
   timer->start( 2000, true );
   QDir dataDir;
   if( dataDir.exists( path ) )
      applyOptions();
   else
   {
      KIO::Job *mkdir = KIO::mkdir( path );
      connect( mkdir, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotMkDirFinish( KIO::Job* ) ) );
   }
}

void BPConf::slotMkDirFinish( KIO::Job *job )
{
   if( job->error() > 0 )
   {
      m_timeOut = FINISH;
      job->showErrorDialog( );
      m_pMain->setEnabled( true );
      return;
   }
   else
      applyOptions();
}

void BPConf::slotReset()
{
   loadOptions( instance()->config() );
   m_pFeeds->clearSelection();
   slotFeedSelect();
}

void BPConf::slotCronData( KProcess*, char *buffer, int bufLen )
{
   m_cronData = QString::fromLatin1( buffer, bufLen );
}

void BPConf::slotCronDataFinish( KProcess* )
{
   m_timeOut = INTER;
   QString path = KURLCompletion::replacedPath( m_pDataDir->text(), true );
   QStringList list = QStringList::split( '\n', m_cronData );
   for( unsigned i = 0; i < list.size(); ++i )
   {
      if ( list[i].endsWith( "bashpodder.shell" ) )
      {
         if( list[i].endsWith( path + "bashpodder.shell" ) )
            list[i] = QString::null;
         else
         {
            QString other = list[i].section( ' ', -1 );
            if( KMessageBox::questionYesNo( 0, i18n( "Found another bashpodder cron job.\nDo you wish to keep the job %1 ?" ).arg( other ),
                i18n( "bashpodder cronjob" ) ) == KMessageBox::No )
               list[i] = QString::null;
         }
      }
   }

   QString newJob;
   QTime time = m_pCheckTime->time();
   switch( m_pCheckInterval->currentItem() )
   {
      case 0: // every hour
         newJob = "0 * * * * ";
         break;
      case 1: // every day
         newJob = QString::number( time.minute() ) + " " + QString::number( time.hour() ) + " * * * ";
         break;
      case 2: // every week
         newJob = QString::number( time.minute() ) + " " + QString::number( time.hour() ) + " * * " +
               QString::number( m_pCheckOnDay->currentItem() ) + " ";
         break;
      case 3: // every month
         newJob = QString::number( time.minute() ) + " " + QString::number( time.hour() ) + m_pCheckOnDate->text() + " * * ";

   }
   newJob += path + "bashpodder.shell";

   KTempFile *file = new KTempFile();
   if( file->status() != 0 )
   {
      KMessageBox::sorry( 0, i18n( "Couldn't create a temp file to store cron job.\n" ) );
      m_pMain->setEnabled( true );
      return;
   }

   QTextStream *stream = file->textStream();
   *stream << newJob << endl;
   QString line;
   for( unsigned i = 0; i < list.size(); ++i )
   {
      if( !list[i].isEmpty() )
         *stream << list[i] << endl;
   }
   file->close();

   m_timeOut = CRONREPLACE;
   KProcess *proc = new KProcess;
   *proc << "crontab" << file->name();
   connect( proc, SIGNAL( processExited( KProcess* ) ), SLOT( slotCronReplaceFinish( KProcess* ) ) );
   proc->start();
   setModified( false );
   saveOptions( instance()->config() );
   m_pMain->setEnabled( true );
}

void BPConf::slotCronReplaceFinish( KProcess* )
{
   m_timeOut = FINISH;
   if( m_checkNow )
      slotCheckNow();
}

void BPConf::slotTimeOut()
{
   switch( m_timeOut )
   {
      case FINISH:
         break;
      case INTER:
         {
            QTimer *timer = new QTimer( this );
            connect( timer, SIGNAL( timeout() ), SLOT( slotTimeOut() ) );
            timer->start( 2000, true );
         }
         break;
      case MKDIR:
         KMessageBox::error( 0, i18n( "Could not make directory %1.\nOperation timed out." ).arg( m_pDataDir->text() ), i18n( "Make Directory" ) );
         break;
      case CRONLIST:
         KMessageBox::error( 0, i18n( "Could not get find crontab.\nPlease check to make sure crontab is installed properly" ), i18n( "Crontab list" ) );
         m_timeOut = FINISH;
         m_pMain->setEnabled( true );
         break;
      case CRONREPLACE:
         KMessageBox::error( 0, i18n( "Could not replace crontab." ), i18n( "Crontab Replace" ) );
         m_timeOut = FINISH;
         m_pMain->setEnabled( true );
         break;
   }
}

void BPConf::setupActions()
{
   KStdAction::openNew( this, SLOT(fileNew()), actionCollection() );
   KStdAction::quit( this, SLOT(close()), actionCollection() );
   new KAction( i18n( "Import configuration file..." ), "fileopen", 0, this, SLOT(importConfig()), actionCollection(), "import_config" );
   new KAction( i18n( "Export configuration file..." ), "filesaveas", 0, this, SLOT(exportConfig()), actionCollection(), "export_config" );
}

void BPConf::setModified( bool modified )
{
   m_modified = modified;
   m_pApply->setEnabled( modified );
   m_pReset->setEnabled( modified );
}

void BPConf::loadOptions( KConfig *config )
{
   config->setGroup( "options" );
   m_pDataDir->setText( config->readEntry( "data_dir", "~/PodCasts/" ) );
   m_pCheckInterval->setCurrentItem( config->readNumEntry( "check_interval", 1 ) );
   slotInterval( m_pCheckInterval->currentItem() );
   m_pCheckOnDay->setCurrentItem( config->readNumEntry( "check_on_day" ) );
   m_pCheckOnDate->setText( config->readEntry( "check_on_date" ) );
   m_pCheckTime->setTime( QTime::fromString( config->readEntry( "check_time", "00:00" ) ) );
   int numFeeds = config->readNumEntry( "number_of_feeds" );
   QString feedtag;
   m_pFeeds->clear();
   for( int i = 0; i < numFeeds; ++i )
   {
      feedtag = "feed" + QString::number( i );
      new QListViewItem( m_pFeeds, config->readEntry( feedtag + "_name" ), config->readEntry( feedtag + "_url" ) );
   }
   m_modified = config->readBoolEntry( "modified", false );
   setModified( m_modified );
}

void BPConf::saveOptions( KConfig *config )
{
   config->deleteGroup( "options" );
   config->setGroup( "options" );
   config->writeEntry( "data_dir", m_pDataDir->text() );
   config->writeEntry( "check_interval", m_pCheckInterval->currentItem() );
   config->writeEntry( "check_on_day", m_pCheckOnDay->currentItem() );
   config->writeEntry( "check_on_date", m_pCheckOnDate->text() );
   config->writeEntry( "check_time", m_pCheckTime->time().toString() );
   int numFeeds = m_pFeeds->childCount();
   config->writeEntry( "number_of_feeds", numFeeds );
   QString feedtag;
   for( int i = 0; i < numFeeds; ++i )
   {
      feedtag = "feed" + QString::number( i );
      QListViewItem *item = m_pFeeds->itemAtIndex( i );
      config->writeEntry( feedtag + "_name", item->text( 0 ) );
      config->writeEntry( feedtag + "_url", item->text( 1 ) );
   }
   config->writeEntry( "modified", m_modified );
}

void BPConf::loadFeeds( const QString &fileName )
{
   QFile file( fileName );
   if( file.open( IO_ReadOnly ) )
   {
      QString line, name;
      QTextStream stream( &file );
      int num = 0;
      while( !stream.atEnd() )
      {
         line = stream.readLine().simplifyWhiteSpace();
         if( num < 10 )
            name = QString( "Imported 0%1" ).arg( num++ );
         else
            name = QString( "Imported %1" ).arg( num++ );
         new QListViewItem( m_pFeeds, name, line );
      }
      file.close();
      setModified( true );
   }
   else
      KMessageBox::sorry( 0, i18n( "I couldn't open the file %1" ).arg( fileName ), i18n( "Couldn't Open" ) );
}

bool BPConf::saveFeeds( const QString &fileName )
{
   QFile file( fileName );
   if( file.open( IO_WriteOnly ) )
   {
      QTextStream stream( &file );

      int numFeeds = m_pFeeds->childCount();
      for( int i = 0; i < numFeeds; ++i )
         stream << m_pFeeds->itemAtIndex( i )->text( 1 ) << "\n";

      file.close();
      return true;
   }
   else
   {
      KMessageBox::sorry( 0, i18n( "Sorry, I couldn't save the file %1" ).arg( fileName ), i18n( "Couldn't save" ) );
      return false;
   }
}

void BPConf::applyOptions()
{
   m_timeOut = INTER;
   QString path = KURLCompletion::replacedPath( m_pDataDir->text(), true );
   if( !saveFeeds( path + "bp.conf" ) )
   {
      m_pMain->setEnabled( true );
      return;
   }

   if( !saveBashFile( path ) )
   {
      m_pMain->setEnabled( true );
      return;
   }

   m_timeOut = CRONLIST;
   m_cronData = QString::null;
   KProcess *proc = new KProcess;
   *proc << "crontab" << "-l";
   connect( proc, SIGNAL( processExited( KProcess* ) ), SLOT( slotCronDataFinish( KProcess* ) ) );
   connect( proc, SIGNAL( receivedStdout( KProcess*, char*, int ) ), SLOT( slotCronData( KProcess*, char*, int ) ) );
   proc->start( KProcess::NotifyOnExit, KProcess::All );
}

bool BPConf::saveBashFile( const QString &path )
{
   QString fileName = path + "bashpodder.shell";
   if( !QFile::exists( fileName ) )
   {
      QFile file( fileName );
      if( file.open( IO_WriteOnly ) )
      {
         QTextStream stream( &file );
         QString doc =
               "#!/bin/bash\n"
               "# By Linc 10/1/2004\n"
               "# Find the latest script at http://linc.homeunix.org:8080/scripts/bashpodder\n"
               "# Last revision 11/17/2004 - Many Contributers!\n"
               "# If you use this and have made improvements or have comments\n"
               "# drop me an email at linc dot fessenden at gmail dot com\n"
               "# I'd appreciate it!\n\n"
               "# Make script crontab friendly:\n"
               "cd $(dirname $0)\n\n"
               "# datadir is the directory you want podcasts saved to:\n"
               "datadir=$(date +%Y-%m-%d)\n\n"
               "# Check for and create datadir if necessary:\n"
               "if test ! -d $datadir\n"
               "\tthen\n"
               "\tmkdir $datadir\n"
               "fi\n\n"
               "# Delete any temp file:\n"
               "rm -f temp.log\n\n"
               "# Read the bp.conf file and wget any url not already in the podcast.log file:\n"
               "while read podcast\n"
               "\tdo\n"
               "\tfile=$(wget -q $podcast -O - | tr '\\r' '\\n' | tr \\' \\\" | sed -n 's/.*url=\"\\([^\"]*\\)\".*/\\1/p')\n"
               "\tfor url in $file\n"
               "\t\tdo\n"
               "\t\techo $url >> temp.log\n"
               "\t\tif ! grep \"$url\" podcast.log > /dev/null\n"
               "\t\t\tthen\n"
               "\t\t\twget -q -P $datadir \"$url\"\n"
               "\t\tfi\n"
               "\t\tdone\n"
               "\tdone < bp.conf\n"
               "# Move dynamically created log file to permanent log file:\n"
               "mv temp.log podcast.log\n"
               "# Create an m3u playlist:\n"
               "ls $datadir | grep -v m3u > $datadir/podcast.m3u\n";
         stream << doc;

         file.close();

         KIO::chmod( fileName, 0755 );
      }
      else
      {
         KMessageBox::sorry( 0, i18n( "Sorry, I couldn't save the bashpodder.shell file." ), i18n( "Couldn't save bashpodder.shell" ) );
         return false;
      }
   }
   return true;
}

#include "bpconf.moc"
