/***************************************************************************
 *   Copyright (C) 2004 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _BPCONF_H_
#define _BPCONF_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qvaluevector.h>
#include <kmainwindow.h>

// Foward Declarations
class KLineEdit;
class KPushButton;
class KURLCompletion;
class KListView;
class QComboBox;
class QLabel;
class KTimeWidget;
class KConfig;
class KProcess;
namespace KIO{ class Job; };

/**
 * @short Application Main Window
 */
class BPConf : public KMainWindow
{
    Q_OBJECT
public:
    /// Default Constructor
    BPConf();
    /// Default Destructor
    virtual ~BPConf() {}

protected:
   /**
    * This method is called when it is time for the app to save its
    * properties for session management purposes.
    */
   void saveProperties(KConfig *config );
   /**
    * This method is called when this app is restored.  The KConfig
    * object points to the session management config file that was saved
    * with @ref saveProperties
    */
   void readProperties(KConfig *config );

   /// Called when the window is about to close, calls the parts queryClose
   bool queryClose();

protected slots:
   /// Called when new is selected
   void fileNew();
   /// Called when import config is selected
   void importConfig();
   /// Called when export config is selected
   void exportConfig();

   /// Called when a feed is selected
   void slotFeedSelect();
   /// Called when the add feed button is pressed
   void slotAddFeed();
   /// Called when the modify feed button is pressed
   void slotModFeed();
   /// Called when the remove feed button is pressed
   void slotRemFeed();

   /// Called when the data dir button is pressed
   void slotDataDir();
   /// Called when the data changes
   void slotDataChange() { setModified( true ); }
   /// Called when the interval changes
   void slotInterval( int item );

   /// Called when the check now button is pressed
   void slotCheckNow();
   /// Called when bashpodder has finished checking
   void slotCheckNowFinish( KProcess* );

   /// Called when the apply button is pressed
   void slotApply();
   /// Called when the reset button is pressed
   void slotReset();

   /// Called when the mkdir has finished
   void slotMkDirFinish( KIO::Job *job );
   /// Called when the crontab returns data
   void slotCronData( KProcess*, char *buffer, int bufLen );
   /// Called when the crontab data finishes
   void slotCronDataFinish( KProcess* );
   /// Called when the crontab replace finishes
   void slotCronReplaceFinish( KProcess* );

   /// Called if there is a timeout
   void slotTimeOut();

private:
   /// Sets the actions for the program
   void setupActions();

   /// Loads the options from a config
   void loadOptions( KConfig *config );
   /// Saves the options from a config
   void saveOptions( KConfig *config );

   /// Loads the feeds from a bashpodder file
   void loadFeeds( const QString &fileName );
   /// Save the feeds to a bashpodder file
   bool saveFeeds( const QString &fileName );

   /// Saves the bp.conf file, the bashfile, and sets up the cronJob
   void applyOptions();
   /// saves the bash file to the specified path
   bool saveBashFile( const QString &path );

   /// Holds whether the configuration has been modified
   bool m_modified;
   /// set whether the configuration is modified
   void setModified( bool modified );

   /// Holds whether bashpodder is currently checking
   bool m_checkNow;
   enum { FINISH, INTER, MKDIR, CRONLIST, CRONREPLACE } m_timeOut;

   // Widgets and UI elements
   QWidget *m_pMain;
   KURLCompletion *m_pDirComplete;
   KLineEdit *m_pDataDir;
   KPushButton *m_pDataDirButton;
   KListView *m_pFeeds;
   KPushButton *m_pFeedAdd;
   KPushButton *m_pFeedMod;
   KPushButton *m_pFeedRem;
   QComboBox *m_pCheckInterval;
   QLabel *m_pCheckOnLabel;
   QComboBox *m_pCheckOnDay;
   KLineEdit *m_pCheckOnDate;
   QLabel *m_pCheckAtLabel;
   KTimeWidget *m_pCheckTime;
   KPushButton *m_pCheckNow;
   KPushButton *m_pApply;
   KPushButton *m_pReset;

   QString m_cronData;
};

#endif // _BPCONF_H_
